#!/usr/bin/perl -T
use strict;
use warnings;
use Impedance::Model;
use Impedance::IMPCGI qw(:html :cgi);
use CGI qw(:standard :netscape);
if (param('action'))
{
    my $data = param('data');
    my $fdata = param('filedata');
    my $model = Impedance::IMPCGI::format_model(param('model'));
    my $m = Impedance::Model->new();
    $m->parse($model);
    my $minf = param('minf');
    my $maxf = param('maxf');
    $minf = 1 unless ($minf);
    $maxf = 100000 unless ($maxf);
    my $text = param('text');
    my $name = param('filename');
    unless ($name)
    {
	my $txt = $m->model_text();
	$txt =~ s/\s+\=\s+/\=/;
	$txt =~ s/[\s\t]+/\-/g;
	$name = time.'_Simulation_'.$txt; 
    }
    my $fit = param('fit_res');
    my $mode = param('mode');
    $data = '' unless (defined($data));
    $fdata = '' unless (defined($fdata));
    $text = '' unless (defined($text));
    $fit = '' unless (defined($fit));
    my $time = localtime();
    my @out = ();
    push @out,"\n\n####  $time  ####\n\n";
    push @out,'### File ###';
    push @out,$name if ($name);
    push @out,'### Data ###';
    push @out,$data if ($data);
    push @out,'### Simulated data (model) ###';
    if ($data)
    {
	foreach my $line (split(/\n/,$data))
	{
	    next unless ($line);
	    my ($f,$r,$i) = split(/[\s\t]+/,$line);
	    next unless ($f);
	    my $z = $m->F($f);
	    push @out,join(' ',$f,sprintf("%.6g",$z->Re),sprintf("%.6g",$z->Im));
	}
    }
    else
    {
	for (my $x = log($maxf)/log(10); $x >= log($minf)/log(10); $x-= 0.05)
	{
	    my $f = 10**$x;
	    my $z = $m->F($f);
	    push @out,join(' ',$f,sprintf("%.6g",$z->Re),sprintf("%.6g",$z->Im));
	}	
    }
    push @out,'### Additional text ###';
    push @out,$text if ($text);
    push @out,'### Impedance model ###';
    push @out,$m->print_model();
    push @out,'### File content ###';
    push @out,$fdata;
    push @out,'### Fit result ###';
    push @out,$fit if ($fit);
    push @out,'### GNUPLOT Commands ###';
    push @out,"# Frequency range: $minf - $maxf Hz";
    if ($mode && $mode =~ /bode/i)
    {
	push @out, $m->print_bode_sim($minf,$maxf);
    }
    else
    {
	push @out, $m->print_imp_sim($minf,$maxf);
    }
    push @out,'### END ###';
    foreach my $l (@out)
    {
	next unless (defined($l));
	$l =~ s/\n/\r\n/g;
    }
    print header(-Content_disposition=>"attachment; filename=$name.ea",-Content_type=>'application/octet-stream',-onblur=>'self.close()');
    print join("\r\n",@out),"\r\n";
    exit;
}
else
{
    &print_header("Elchemea Analytical",-onBlur=>'initiateSelfClosing()',-onload=>'load_data()');

    print start_form(-name=>'sofc');
    print hidden(-name=>'text');
    print hidden(-name=>'model');
    print hidden(-name=>'data',-value=>'',-force=>1);
    print hidden(-name=>'filename');
    print hidden(-name=>'filedata');
    print hidden(-name=>'fit_res');
    print hidden(-name=>'allow_close',-value=>1,-force=>1);
    print hidden(-name=>'action',-value=>'dummy',-force=>1);

    print end_form();
    print <<END;

<script language="javascript">
function load_data()
{
    var s = opener.document.sofc;
    var t = document.sofc;
    if (s != undefined)
    {
	t.text.value = s.text.value;
	t.model.value = s.model.value;
	t.data.value = s.data.value;
	t.filename.value = s.filename.value;
	t.filedata.value = s.filedata.value;
	t.fit_res.value = s.fit_res.value;
    }
    t.submit();
}
</script>

END

    print end_html;
}
